@@ -71,7 +71,7 @@ const char* get_model_id(vw_ptr all) { return all->id.c_str(); }
 string get_arguments(vw_ptr all)
 {
 	string args;
-	for (auto& s : all->args)
+	for (auto& s : all->opts_n_args.args)
 	{
 		args.append(s);
 		args.append(" ");
@@ -168,31 +168,52 @@ void my_finish_example(vw_ptr all, example_ptr ec)
 
 void my_learn(vw_ptr all, example_ptr ec)
 { if (ec->test_only)
-  { all->l->predict(*ec);
+  { as_singleline(all->l)->predict(*ec);
   }
   else
-  { all->learn(ec.get());
+  { all->learn(*ec.get());
   }
 }
 
 float my_learn_string(vw_ptr all, char*str)
 { example*ec = VW::read_example(*all, str);
-  all->learn(ec);
+  all->learn(*ec);
   float pp = ec->partial_prediction;
-  VW::finish_example(*all, ec);
+  VW::finish_example(*all, *ec);
   return pp;
 }
 
 float my_predict(vw_ptr all, example_ptr ec)
-{ all->l->predict(*ec);
+{ as_singleline(all->l)->predict(*ec);
   return ec->partial_prediction;
 }
 
+template<bool learn>
+void predict_or_learn(vw_ptr& all, py::list& ec)
+{ multi_ex ex_coll;
+  for (ssize_t i = 0; i<len(ec); i++)
+  { py::object eci = ec[i];
+    py::extract<example_ptr> get_ex(eci);
+    example_ptr ecp;
+    if (get_ex.check())
+      ecp = get_ex();
+    ex_coll.push_back(ecp.get());
+  }
+  if (learn) all->learn(ex_coll);
+  else as_multiline(all->l)->predict(ex_coll);
+}
+
+void my_learn_multi_ex(vw_ptr& all, py::list& ec)
+{  predict_or_learn<true>(all, ec); }
+
+void my_predict_multi_ex(vw_ptr& all, py::list& ec)
+{ predict_or_learn<false>(all, ec); }
+
 float my_predict_string(vw_ptr all, char*str)
 { example*ec = VW::read_example(*all, str);
-  all->l->predict(*ec);
+  as_singleline(all->l)->predict(*ec);
   float pp = ec->partial_prediction;
-  VW::finish_example(*all, ec);
+  VW::finish_example(*all, *ec);
   return pp;
 }
 
@@ -330,7 +351,7 @@ void ex_erase_namespace(example_ptr ec, unsigned char ns)
 { ec->num_features -= ec->feature_space[ns].size();
   ec->total_sum_feat_sq -= ec->feature_space[ns].sum_feat_sq;
   ec->feature_space[ns].sum_feat_sq = 0.;
-  ec->feature_space[ns].erase();
+  ec->feature_space[ns].clear();
 }
 
 bool ex_pop_namespace(example_ptr ec)
@@ -362,7 +383,7 @@ void unsetup_example(vw_ptr vwP, example_ptr ae)
   }
 
   if (all.add_constant)
-  { ae->feature_space[constant_namespace].erase();
+  { ae->feature_space[constant_namespace].clear();
     int hit_constant = -1;
     size_t N = ae->indices.size();
     for (size_t i=0; i<N; i++)
@@ -593,34 +614,34 @@ void my_set_test_only(example_ptr ec, bool val) { ec->test_only = val; }
 
 bool po_exists(search_ptr sch, string arg)
 { HookTask::task_data* d = sch->get_task_data<HookTask::task_data>();
-  return (*d->var_map).count(arg) > 0;
+  return d->arg->vm.count(arg) > 0;
 }
 
 string po_get_string(search_ptr sch, string arg)
 { HookTask::task_data* d = sch->get_task_data<HookTask::task_data>();
-  return (*d->var_map)[arg].as<string>();
+  return d->arg->vm[arg].as<string>();
 }
 
 int32_t po_get_int(search_ptr sch, string arg)
 { HookTask::task_data* d = sch->get_task_data<HookTask::task_data>();
-  try { return (*d->var_map)[arg].as<int>(); }
+  try { return d->arg->vm[arg].as<int>(); }
   catch (...) {}
-  try { return (int32_t)(*d->var_map)[arg].as<size_t>(); }
+  try { return (int32_t)d->arg->vm[arg].as<size_t>(); }
   catch (...) {}
-  try { return (int32_t)(*d->var_map)[arg].as<uint32_t>(); }
+  try { return (int32_t)d->arg->vm[arg].as<uint32_t>(); }
   catch (...) {}
-  try { return (int32_t)(*d->var_map)[arg].as<uint64_t>(); }
+  try { return (int32_t)d->arg->vm[arg].as<uint64_t>(); }
   catch (...) {}
-  try { return (*d->var_map)[arg].as<uint16_t>(); }
+  try { return d->arg->vm[arg].as<uint16_t>(); }
   catch (...) {}
-  try { return (*d->var_map)[arg].as<int32_t>(); }
+  try { return d->arg->vm[arg].as<int32_t>(); }
   catch (...) {}
-  try { return (int32_t)(*d->var_map)[arg].as<int64_t>(); }
+  try { return (int32_t)d->arg->vm[arg].as<int64_t>(); }
   catch (...) {}
-  try { return (int32_t)(*d->var_map)[arg].as<int16_t>(); }
+  try { return (int32_t)d->arg->vm[arg].as<int16_t>(); }
   catch (...) {}
   // we know this'll fail but do it anyway to get the exception
-  return (*d->var_map)[arg].as<int>();
+  return d->arg->vm[arg].as<int>();
 }
 
 PyObject* po_get(search_ptr sch, string arg)
@@ -692,6 +713,9 @@ BOOST_PYTHON_MODULE(pylibvw)
   .def("get_id", &get_model_id, "return the model id")
   .def("get_arguments", &get_arguments, "return the arguments after resolving all dependencies")
 
+  .def("learn_multi", &my_learn_multi_ex, "given a list pyvw examples, learn (and predict) on those examples")
+  .def("predict_multi", &my_predict_multi_ex, "given a list of pyvw examples, predict on that example")
+
   .def_readonly("lDefault", lDEFAULT, "Default label type (whatever vw was initialized with) -- used as input to the example() initializer")
   .def_readonly("lBinary", lBINARY, "Binary label type -- used as input to the example() initializer")
   .def_readonly("lMulticlass", lMULTICLASS, "Multiclass label type -- used as input to the example() initializer")
@@ -706,7 +730,7 @@ BOOST_PYTHON_MODULE(pylibvw)
   .def_readonly("pMULTILABELS", pMULTILABELS, "Multilabel prediction type")
   .def_readonly("pPROB", pPROB, "Probability prediction type")
   .def_readonly("pMULTICLASSPROBS", pMULTICLASSPROBS, "Multiclass probabilities prediction type")
-  ;
+;
 
   // define the example class
   py::class_<example, example_ptr>("example", py::no_init)
