@@ -3,7 +3,7 @@
 
 using namespace std;
 template <bool is_learn>
-void predict_or_learn(char&, LEARNER::base_learner& base, example& ec)
+void predict_or_learn(char&, LEARNER::single_learner& base, example& ec)
 {
   if (is_learn)
     base.learn(ec);
@@ -26,13 +26,13 @@ void predict_or_learn(char&, LEARNER::base_learner& base, example& ec)
   }
 }
 
-LEARNER::base_learner* binary_setup(vw& all)
+LEARNER::base_learner* binary_setup(arguments& arg)
 {
-  if (missing_option(all, false, "binary", "report loss as binary classification on -1,1"))
+  if (arg.new_options("Binary loss").
+      critical("binary", "report loss as binary classification on -1,1").missing())
     return nullptr;
 
-  LEARNER::learner<char>& ret =
-    LEARNER::init_learner<char>(nullptr, setup_base(all),
-                                predict_or_learn<true>, predict_or_learn<false>);
+  LEARNER::learner<char,example>& ret =
+    LEARNER::init_learner(as_singleline(setup_base(arg)), predict_or_learn<true>, predict_or_learn<false>);
   return make_base(ret);
 }
