@@ -6,6 +6,9 @@ license as described in the file LICENSE.
 #pragma once
 
 #include "label_parser.h"
+#include<vector>
+
+struct example;
 
 namespace CB
 {
@@ -17,17 +20,12 @@ struct cb_class
   bool operator==(cb_class j) {return action == j.action;}
 };
 
-struct label
-{ v_array<cb_class> costs;
-};
-
-bool is_test_label(CB::label& ld);
+struct label { v_array<cb_class> costs;};
 
 extern label_parser cb_label;//for learning
-bool example_is_test(example& ec);
 bool ec_is_example_header(example& ec);  // example headers look like "0:-1" or just "shared"
 
-void print_update(vw& all, bool is_test, example& ec, v_array<example*>* ec_seq, bool multilabel);
+ void print_update(vw& all, bool is_test, example& ec, std::vector<example*>* ec_seq, bool action_scores);
 }
 
 namespace CB_EVAL
