@@ -5,9 +5,12 @@ license as described in the file LICENSE.
 */
 #pragma once
 
-LEARNER::base_learner* cb_explore_adf_setup(vw& all);
+#include<vector>
+struct example;
+
+LEARNER::base_learner* cb_explore_adf_setup(arguments& arg);
 
 namespace CB_EXPLORE_ADF
 {
-example* test_adf_sequence(v_array<example*>& ec_seq);
+  example* test_adf_sequence(std::vector<example*>& ec_seq);
 }
