@@ -3,5 +3,4 @@ Copyright (c) by respective owners including Yahoo!, Microsoft, and
 individual contributors. All rights reserved.  Released under a BSD
 license as described in the file LICENSE.
  */
-LEARNER::base_learner* cbify_setup(vw& all);
-
+LEARNER::base_learner* cbify_setup(arguments& arg);
