@@ -27,6 +27,7 @@ license as described in the file LICENSE.
 #include "vw_allreduce.h"
 #include "rand48.h"
 #include "floatbits.h"
+#include "reductions.h"
 
 #define SVM_KER_LIN 0
 #define SVM_KER_RBF 1
@@ -156,7 +157,7 @@ make_hot_sv(svm_params& params, size_t svi)
   svm_model *model = params.model;
   size_t n = model->num_support;
   if (svi >= model->num_support)
-    params.all->trace_message << "Internal error at " << __FILE__ << ":" << __LINE__ << endl;
+    params.all->opts_n_args.trace_message << "Internal error at " << __FILE__ << ":" << __LINE__ << endl;
   // rotate params fields
   svm_example *svi_e = model->support_vec[svi];
   int alloc = svi_e->compute_kernels(params);
@@ -279,12 +280,12 @@ void save_load_svm_model(svm_params& params, io_buf& model_file, bool read, bool
   svm_model* model = params.model;
   //TODO: check about initialization
 
-  //params.all->trace_message<<"Save load svm "<<read<<" "<<text<<endl;
+  //params.all->opts_n_args.trace_message<<"Save load svm "<<read<<" "<<text<<endl;
   if (model_file.files.size() == 0) return;
   stringstream msg;
   bin_text_read_write_fixed(model_file,(char*)&(model->num_support), sizeof(model->num_support),
                             "", read, msg, text);
-  //params.all->trace_message<<"Read num support "<<model->num_support<<endl;
+  //params.all->opts_n_args.trace_message<<"Read num support "<<model->num_support<<endl;
 
   flat_example* fec = nullptr;
   if(read)
@@ -320,7 +321,7 @@ void save_load(svm_params& params, io_buf& model_file, bool read, bool text)
 {
   if(text)
   {
-    params.all->trace_message<<"Not supporting readable model for kernel svm currently"<<endl;
+    params.all->opts_n_args.trace_message<<"Not supporting readable model for kernel svm currently"<<endl;
     return;
   }
 
@@ -342,7 +343,7 @@ float linear_kernel(const flat_example* fec1, const flat_example* fec2)
   {
     uint64_t ec1pos = fs_1.indicies[idx1];
     uint64_t ec2pos = fs_2.indicies[idx2];
-    //params.all->trace_message<<ec1pos<<" "<<ec2pos<<" "<<idx1<<" "<<idx2<<" "<<f->x<<" "<<ec2f->x<<endl;
+    //params.all->opts_n_args.trace_message<<ec1pos<<" "<<ec2pos<<" "<<idx1<<" "<<idx2<<" "<<f->x<<" "<<ec2f->x<<endl;
     if(ec1pos < ec2pos) continue;
 
     while(ec1pos > ec2pos && ++idx2 < fs_2.size())
@@ -350,14 +351,14 @@ float linear_kernel(const flat_example* fec1, const flat_example* fec2)
 
     if(ec1pos == ec2pos)
     {
-      //params.all->trace_message<<ec1pos<<" "<<ec2pos<<" "<<idx1<<" "<<idx2<<" "<<f->x<<" "<<ec2f->x<<endl;
+      //params.all->opts_n_args.trace_message<<ec1pos<<" "<<ec2pos<<" "<<idx1<<" "<<idx2<<" "<<f->x<<" "<<ec2f->x<<endl;
       numint++;
       dotprod += fs_1.values[idx1] * fs_2.values[idx2];
       ++idx2;
     }
   }
-  //params.all->trace_message<<endl;
-  //params.all->trace_message<<"numint = "<<numint<<" dotprod = "<<dotprod<<endl;
+  //params.all->opts_n_args.trace_message<<endl;
+  //params.all->opts_n_args.trace_message<<"numint = "<<numint<<" dotprod = "<<dotprod<<endl;
   return dotprod;
 }
 
@@ -399,7 +400,7 @@ float dense_dot(float* v1, v_array<float> v2, size_t n)
 }
 
 
-void predict (svm_params& params, svm_example** ec_arr, float* scores, size_t n)
+void predict(svm_params& params, svm_example** ec_arr, float* scores, size_t n)
 {
   svm_model* model = params.model;
   for(size_t i = 0; i < n; i++)
@@ -409,7 +410,7 @@ void predict (svm_params& params, svm_example** ec_arr, float* scores, size_t n)
   }
 }
 
-void predict(svm_params& params, base_learner &, example& ec)
+void predict(svm_params& params, single_learner &, example& ec)
 {
   flat_example* fec = flatten_sort_example(*(params.all),&ec);
   if(fec)
@@ -433,7 +434,7 @@ size_t suboptimality(svm_model* model, double* subopt)
   double max_val = 0;
   for(size_t i = 0; i < model->num_support; i++)
   {
-    double tmp = model->alpha[i]*model->support_vec[i]->ex.l.simple.label;
+    float tmp = model->alpha[i]*model->support_vec[i]->ex.l.simple.label;
 
     if((tmp < model->support_vec[i]->ex.l.simple.weight && model->delta[i] < 0) || (tmp > 0 && model->delta[i] > 0))
       subopt[i] = fabs(model->delta[i]);
@@ -455,7 +456,7 @@ int remove(svm_params& params, size_t svi)
 {
   svm_model* model = params.model;
   if (svi >= model->num_support)
-    params.all->trace_message << "Internal error at " << __FILE__ << ":" << __LINE__ << endl;
+    params.all->opts_n_args.trace_message << "Internal error at " << __FILE__ << ":" << __LINE__ << endl;
   // shift params fields
   svm_example* svi_e = model->support_vec[svi];
   for (size_t i=svi; i<model->num_support-1; ++i)
@@ -500,12 +501,10 @@ int add(svm_params& params, svm_example* fec)
 
 bool update(svm_params& params, size_t pos)
 {
-
-  //params.all->trace_message<<"Update\n";
+  //params.all->opts_n_args.trace_message<<"Update\n";
   svm_model* model = params.model;
   bool overshoot = false;
-  //params.all->trace_message<<"Updating model "<<pos<<" "<<model->num_support<<" ";
-  //params.all->trace_message<<model->support_vec[pos]->example_counter<<endl;
+  //params.all->opts_n_args.trace_message<<"Updating model "<<pos<<" "<<model->num_support<<" ";
   svm_example* fec = model->support_vec[pos];
   label_data& ld = fec->ex.l.simple;
   fec->compute_kernels(params);
@@ -518,7 +517,7 @@ bool update(svm_params& params, size_t pos)
 
   float proj = alphaKi*ld.label;
   float ai = (params.lambda - proj)/inprods[pos];
-  //params.all->trace_message<<model->num_support<<" "<<pos<<" "<<proj<<" "<<alphaKi<<" "<<alpha_old<<" "<<ld->label<<" "<<model->delta[pos]<<" ";
+  //params.all->opts_n_args.trace_message<<model->num_support<<" "<<pos<<" "<<proj<<" "<<alphaKi<<" "<<alpha_old<<" "<<ld.label<<" "<<model->delta[pos]<<" " << endl;
 
   if(ai > fec->ex.l.simple.weight)
     ai = fec->ex.l.simple.weight;
@@ -533,7 +532,7 @@ bool update(svm_params& params, size_t pos)
 
   if(fabs(diff) > 1.)
   {
-    //params.all->trace_message<<"Here\n";
+    //params.all->opts_n_args.trace_message<<"Here\n";
     diff = (float) (diff > 0) - (diff < 0);
     ai = alpha_old + diff;
   }
@@ -588,7 +587,7 @@ void sync_queries(vw& all, svm_params& params, bool* train_pool)
 
   size_t* sizes = calloc_or_throw<size_t>(all.all_reduce->total);
   sizes[all.all_reduce->node] = b->head - b->space.begin();
-  //params.all->trace_message<<"Sizes = "<<sizes[all.node]<<" ";
+  //params.all->opts_n_args.trace_message<<"Sizes = "<<sizes[all.node]<<" ";
   all_reduce<size_t, add_size_t>(all, sizes, all.all_reduce->total);
 
   size_t prev_sum = 0, total_sum = 0;
@@ -599,7 +598,7 @@ void sync_queries(vw& all, svm_params& params, bool* train_pool)
     total_sum += sizes[i];
   }
 
-  //params.all->trace_message<<total_sum<<" "<<prev_sum<<endl;
+  //params.all->opts_n_args.trace_message<<total_sum<<" "<<prev_sum<<endl;
   if(total_sum > 0)
   {
     queries = calloc_or_throw<char>(total_sum);
@@ -623,8 +622,8 @@ void sync_queries(vw& all, svm_params& params, bool* train_pool)
         train_pool[i] = true;
         params.pool_pos++;
         // for(int j = 0;j < fec->feature_map_len;j++)
-        //   params.all->trace_message<<fec->feature_map[j].weight_index<<":"<<fec->feature_map[j].x<<" ";
-        // params.all->trace_message<<endl;
+        //   params.all->opts_n_args.trace_message<<fec->feature_map[j].weight_index<<":"<<fec->feature_map[j].x<<" ";
+        // params.all->opts_n_args.trace_message<<endl;
         // params.pool[i]->in_use = true;
         // params.current_t += ((label_data*) params.pool[i]->ld)->weight;
         // params.pool[i]->example_t = params.current_t;
@@ -650,7 +649,7 @@ void sync_queries(vw& all, svm_params& params, bool* train_pool)
 void train(svm_params& params)
 {
 
-  //params.all->trace_message<<"In train "<<params.all->training<<endl;
+  //params.all->opts_n_args.trace_message<<"In train "<<params.all->training<<endl;
 
   bool* train_pool = calloc_or_throw<bool>(params.pool_size);
   for(size_t i = 0; i < params.pool_size; i++)
@@ -660,7 +659,6 @@ void train(svm_params& params)
   predict(params, params.pool, scores, params.pool_pos);
   //cout<<scores[0]<<endl;
 
-
   if(params.active)
   {
     if(params.active_pool_greedy)
@@ -670,22 +668,21 @@ void train(svm_params& params)
         scoremap.insert(pair<const double, const size_t>(fabs(scores[i]),i));
 
       multimap<double, size_t>::iterator iter = scoremap.begin();
-      //params.all->trace_message<<params.pool_size<<" "<<"Scoremap: ";
+      //params.all->opts_n_args.trace_message<<params.pool_size<<" "<<"Scoremap: ";
       //for(;iter != scoremap.end();iter++)
-      //params.all->trace_message<<iter->first<<" "<<iter->second<<" "<<((label_data*)params.pool[iter->second]->ld)->label<<"\t";
-      //params.all->trace_message<<endl;
+      //params.all->opts_n_args.trace_message<<iter->first<<" "<<iter->second<<" "<<((label_data*)params.pool[iter->second]->ld)->label<<"\t";
+      //params.all->opts_n_args.trace_message<<endl;
       iter = scoremap.begin();
 
       for(size_t train_size = 1; iter != scoremap.end() && train_size <= params.subsample; train_size++)
       {
-        //params.all->trace_message<<train_size<<" "<<iter->second<<" "<<iter->first<<endl;
+        //params.all->opts_n_args.trace_message<<train_size<<" "<<iter->second<<" "<<iter->first<<endl;
         train_pool[iter->second] = 1;
         iter++;
       }
     }
     else
     {
-
       for(size_t i = 0; i < params.pool_pos; i++)
       {
         float queryp = 2.0f/(1.0f + expf((float)(params.active_c*fabs(scores[i]))*(float)pow(params.pool[i]->ex.example_counter,0.5f)));
@@ -711,37 +708,36 @@ void train(svm_params& params)
 
   if(params.all->training)
   {
-
     svm_model* model = params.model;
 
     for(size_t i = 0; i < params.pool_pos; i++)
     {
-      //params.all->trace_message<<"process: "<<i<<" "<<train_pool[i]<<endl;;
+      //params.all->opts_n_args.trace_message<<"process: "<<i<<" "<<train_pool[i]<<endl;;
       int model_pos = -1;
       if(params.active)
       {
         if(train_pool[i])
         {
-          //params.all->trace_message<<"i = "<<i<<"train_pool[i] = "<<train_pool[i]<<" "<<params.pool[i]->example_counter<<endl;
+          //params.all->opts_n_args.trace_message<<"i = "<<i<<"train_pool[i] = "<<train_pool[i]<<" "<<params.pool[i]->example_counter<<endl;
           model_pos = add(params, params.pool[i]);
         }
       }
       else
         model_pos = add(params, params.pool[i]);
 
-      //params.all->trace_message<<"Added: "<<model_pos<<" "<<model->support_vec[model_pos]->example_counter<<endl;
+      //params.all->opts_n_args.trace_message<<"Added: "<<model_pos<<" "<<model->support_vec[model_pos]->example_counter<<endl;
       //cout<<"After adding in train "<<model->num_support<<endl;
 
       if(model_pos >= 0)
       {
         bool overshoot = update(params, model_pos);
-        //params.all->trace_message<<model_pos<<":alpha = "<<model->alpha[model_pos]<<endl;
+        //params.all->opts_n_args.trace_message<<model_pos<<":alpha = "<<model->alpha[model_pos]<<endl;
 
         double* subopt = calloc_or_throw<double>(model->num_support);
         for(size_t j = 0; j < params.reprocess; j++)
         {
           if(model->num_support == 0) break;
-          //params.all->trace_message<<"reprocess: ";
+          //params.all->opts_n_args.trace_message<<"reprocess: ";
           int randi = 1;
           if (merand48(params.all->random_state) < 0.5)
             randi = 0;
@@ -751,9 +747,9 @@ void train(svm_params& params)
             if(subopt[max_pos] > 0)
             {
               if(!overshoot && max_pos == (size_t)model_pos && max_pos > 0 && j == 0)
-                params.all->trace_message<<"Shouldn't reprocess right after process!!!"<<endl;
-              //params.all->trace_message<<max_pos<<" "<<subopt[max_pos]<<endl;
-              // params.all->trace_message<<params.model->support_vec[0]->example_counter<<endl;
+                params.all->opts_n_args.trace_message<<"Shouldn't reprocess right after process!!!"<<endl;
+              //params.all->opts_n_args.trace_message<<max_pos<<" "<<subopt[max_pos]<<endl;
+              // params.all->opts_n_args.trace_message<<params.model->support_vec[0]->example_counter<<endl;
               if(max_pos*model->num_support <= params.maxcache)
                 make_hot_sv(params, max_pos);
               update(params, max_pos);
@@ -765,8 +761,8 @@ void train(svm_params& params)
             update(params, rand_pos);
           }
         }
-        //params.all->trace_message<<endl;
-        // params.all->trace_message<<params.model->support_vec[0]->example_counter<<endl;
+        //params.all->opts_n_args.trace_message<<endl;
+        // params.all->opts_n_args.trace_message<<params.model->support_vec[0]->example_counter<<endl;
         free(subopt);
       }
     }
@@ -776,22 +772,19 @@ void train(svm_params& params)
     for(size_t i = 0; i < params.pool_pos; i++)
       delete params.pool[i];
 
-  // params.all->trace_message<<params.model->support_vec[0]->example_counter<<endl;
+  // params.all->opts_n_args.trace_message<<params.model->support_vec[0]->example_counter<<endl;
   // for(int i = 0;i < params.pool_size;i++)
-  //   params.all->trace_message<<scores[i]<<" ";
-  // params.all->trace_message<<endl;
+  //   params.all->opts_n_args.trace_message<<scores[i]<<" ";
+  // params.all->opts_n_args.trace_message<<endl;
   free(scores);
-  //params.all->trace_message<<params.model->support_vec[0]->example_counter<<endl;
+  //params.all->opts_n_args.trace_message<<params.model->support_vec[0]->example_counter<<endl;
   free(train_pool);
-  //params.all->trace_message<<params.model->support_vec[0]->example_counter<<endl;
+  //params.all->opts_n_args.trace_message<<params.model->support_vec[0]->example_counter<<endl;
 }
 
-void learn(svm_params& params, base_learner&, example& ec)
+void learn(svm_params& params, single_learner&, example& ec)
 {
   flat_example* fec = flatten_sort_example(*(params.all),&ec);
-  // for(int i = 0;i < fec->feature_map_len;i++)
-  //   cout<<i<<":"<<fec->feature_map[i].x<<" "<<fec->feature_map[i].weight_index<<" ";
-  // cout<<endl;
   if(fec)
   {
     svm_example* sec = &calloc_or_throw<svm_example>();
@@ -805,8 +798,8 @@ void learn(svm_params& params, base_learner&, example& ec)
       trim_cache(params);
     if(params.all->training && ec.example_counter % 1000 == 0 && ec.example_counter >= 2)
     {
-      params.all->trace_message<<"Number of support vectors = "<<params.model->num_support<<endl;
-      params.all->trace_message<<"Number of kernel evaluations = "<<num_kernel_evals<<" "<<"Number of cache queries = "<<num_cache_evals<<" loss sum = "<<params.loss_sum<<" "<<params.model->alpha[params.model->num_support-1]<<" "<<params.model->alpha[params.model->num_support-2]<<endl;
+      params.all->opts_n_args.trace_message<<"Number of support vectors = "<<params.model->num_support<<endl;
+      params.all->opts_n_args.trace_message<<"Number of kernel evaluations = "<<num_kernel_evals<<" "<<"Number of cache queries = "<<num_cache_evals<<" loss sum = "<<params.loss_sum<<" "<<params.model->alpha[params.model->num_support-1]<<" "<<params.model->alpha[params.model->num_support-2]<<endl;
     }
     params.pool[params.pool_pos] = sec;
     params.pool_pos++;
@@ -830,7 +823,7 @@ void free_svm_model(svm_model* model)
     // When the call to allocation is replaced by (a) 'new svm_example()' and deallocated using (b) 'operator delete (model->support_vect[i])', the warning goes away.
     // Disable SDL warning.
     //    #pragma warning(disable:6001)
-    free(model->support_vec[i]);
+    free_it(model->support_vec[i]);
     //  #pragma warning(default:6001)
 
     model->support_vec[i] = 0;
@@ -846,133 +839,81 @@ void finish(svm_params& params)
 {
   free(params.pool);
 
-  params.all->trace_message<<"Num support = "<<params.model->num_support<<endl;
-  params.all->trace_message<<"Number of kernel evaluations = "<<num_kernel_evals<<" "<<"Number of cache queries = "<<num_cache_evals<<endl;
-  params.all->trace_message<<"Total loss = "<<params.loss_sum<<endl;
+  params.all->opts_n_args.trace_message<<"Num support = "<<params.model->num_support<<endl;
+  params.all->opts_n_args.trace_message<<"Number of kernel evaluations = "<<num_kernel_evals<<" "<<"Number of cache queries = "<<num_cache_evals<<endl;
+  params.all->opts_n_args.trace_message<<"Total loss = "<<params.loss_sum<<endl;
 
   free_svm_model(params.model);
-  params.all->trace_message<<"Done freeing model"<<endl;
+  params.all->opts_n_args.trace_message<<"Done freeing model"<<endl;
   if(params.kernel_params) free(params.kernel_params);
-  params.all->trace_message<<"Done freeing kernel params"<<endl;
-  params.all->trace_message<<"Done with finish "<<endl;
+  params.all->opts_n_args.trace_message<<"Done freeing kernel params"<<endl;
+  params.all->opts_n_args.trace_message<<"Done with finish "<<endl;
 }
 
-LEARNER::base_learner* kernel_svm_setup(vw &all)
+LEARNER::base_learner* kernel_svm_setup(arguments& arg)
 {
-  if (missing_option(all, true, "ksvm", "kernel svm")) return nullptr;
-  new_options(all, "KSVM options")
-  ("reprocess", po::value<size_t>(), "number of reprocess steps for LASVM")
-  //      ("active", "do active learning")
-  //("active_c", po::value<double>(), "parameter for query prob")
-  ("pool_greedy", "use greedy selection on mini pools")
-  ("para_active", "do parallel active learning")
-  ("pool_size", po::value<size_t>(), "size of pools for active learning")
-  ("subsample", po::value<size_t>(), "number of items to subsample from the pool")
-  ("kernel", po::value<string>(), "type of kernel (rbf or linear (default))")
-  ("bandwidth", po::value<float>(), "bandwidth of rbf kernel")
-  ("degree", po::value<int>(), "degree of poly kernel")
-  ("lambda", po::value<double>(), "saving regularization for test time");
-  add_options(all);
+  auto params = scoped_calloc_or_throw<svm_params>();
+  std::string kernel_type;
+  float bandwidth=1.f;
+  int degree = 2;
+  if (arg.new_options("Kernel SVM").critical("ksvm", "kernel svm")
+      ("reprocess", params->reprocess, (size_t)1, "number of reprocess steps for LASVM")
+      (params->active_pool_greedy, "pool_greedy", "use greedy selection on mini pools")
+      ("para_active", "do parallel active learning")
+      ("pool_size", params->pool_size, (size_t)1, "size of pools for active learning")
+      ("subsample", params->subsample, (size_t)1, "number of items to subsample from the pool")
+      .keep("kernel", kernel_type, (string)"linear", "type of kernel (rbf or linear (default))")
+      .keep("bandwidth", bandwidth, 1.f, "bandwidth of rbf kernel")
+      .keep("degree", degree, 2, "degree of poly kernel")
+      .keep("lambda", params->lambda, "saving regularization for test time").missing())
+    return nullptr;
 
-  po::variables_map& vm = all.vm;
   string loss_function = "hinge";
   float loss_parameter = 0.0;
-  delete all.loss;
-  all.loss = getLossFunction(all, loss_function, (float)loss_parameter);
+  delete arg.all->loss;
+  arg.all->loss = getLossFunction(*arg.all, loss_function, (float)loss_parameter);
 
-  svm_params& params = calloc_or_throw<svm_params>();
-  params.model = &calloc_or_throw<svm_model>();
-  params.model->num_support = 0;
-  //params.curcache = 0;
-  params.maxcache = 1024*1024*1024;
-  params.loss_sum = 0.;
-  params.all = &all;
+  params->model = &calloc_or_throw<svm_model>();
+  params->model->num_support = 0;
+  params->maxcache = 1024*1024*1024;
+  params->loss_sum = 0.;
+  params->all = arg.all;
 
-  if(vm.count("reprocess"))
-    params.reprocess = vm["reprocess"].as<std::size_t>();
-  else
-    params.reprocess = 1;
+  if(arg.vm["active"].as<bool>())
+    params->active = true;
+  if(params->active)
+    params->active_c = 1.;
 
-  if(vm.count("active"))
-    params.active = true;
-  if(params.active)
-  {
-    if(vm.count("active_c"))
-      params.active_c = vm["active_c"].as<double>();
-    else
-      params.active_c = 1.;
-    if(vm.count("pool_greedy"))
-      params.active_pool_greedy = 1;
-    /*if(vm.count("para_active"))
-      params.para_active = 1;*/
-  }
+  params->pool = calloc_or_throw<svm_example*>(params->pool_size);
+  params->pool_pos = 0;
 
-  if(vm.count("pool_size"))
-    params.pool_size = vm["pool_size"].as<std::size_t>();
-  else
-    params.pool_size = 1;
+  if(!arg.vm.count("subsample") && params->para_active)
+    params->subsample = (size_t)ceil(params->pool_size / arg.all->all_reduce->total);
 
-  params.pool = calloc_or_throw<svm_example*>(params.pool_size);
-  params.pool_pos = 0;
-
-  if(vm.count("subsample"))
-    params.subsample = vm["subsample"].as<std::size_t>();
-  else if(params.para_active)
-    params.subsample = (size_t)ceil(params.pool_size / all.all_reduce->total);
-  else
-    params.subsample = 1;
-
-  params.lambda = all.l2_lambda;
-
-  *all.file_options <<" --lambda "<< params.lambda;
-
-  params.all->trace_message<<"Lambda = "<<params.lambda<<endl;
-
-  std::string kernel_type;
-
-  if(vm.count("kernel"))
-    kernel_type = vm["kernel"].as<std::string>();
-  else
-    kernel_type = string("linear");
-
-  *all.file_options <<" --kernel "<< kernel_type;
-
-  params.all->trace_message<<"Kernel = "<<kernel_type<<endl;
+  params->lambda = arg.all->l2_lambda;
+  params->all->opts_n_args.trace_message<<"Lambda = "<<params->lambda<<endl;
+  params->all->opts_n_args.trace_message<<"Kernel = "<<kernel_type<<endl;
 
   if(kernel_type.compare("rbf") == 0)
   {
-    params.kernel_type = SVM_KER_RBF;
-    float bandwidth = 1.;
-    if(vm.count("bandwidth"))
-    {
-      bandwidth = vm["bandwidth"].as<float>();
-      *all.file_options <<" --bandwidth "<<bandwidth;
-    }
-    params.all->trace_message<<"bandwidth = "<<bandwidth<<endl;
-    params.kernel_params = &calloc_or_throw<double>();
-    *((float*)params.kernel_params) = bandwidth;
+    params->kernel_type = SVM_KER_RBF;
+    params->all->opts_n_args.trace_message<<"bandwidth = "<<bandwidth<<endl;
+    params->kernel_params = &calloc_or_throw<double>();
+    *((float*)params->kernel_params) = bandwidth;
   }
   else if(kernel_type.compare("poly") == 0)
   {
-    params.kernel_type = SVM_KER_POLY;
-    int degree = 2;
-    if(vm.count("degree"))
-    {
-      degree = vm["degree"].as<int>();
-      *all.file_options <<" --degree "<<degree;
-    }
-    params.all->trace_message<<"degree = "<<degree<<endl;
-    params.kernel_params = &calloc_or_throw<int>();
-    *((int*)params.kernel_params) = degree;
+    params->kernel_type = SVM_KER_POLY;
+    params->all->opts_n_args.trace_message<<"degree = "<<degree<<endl;
+    params->kernel_params = &calloc_or_throw<int>();
+    *((int*)params->kernel_params) = degree;
   }
   else
-    params.kernel_type = SVM_KER_LIN;
+    params->kernel_type = SVM_KER_LIN;
 
-  //params.all->weights->mask((uint64_t)LONG_MAX);
-  params.all->weights.stride_shift(0);
+  params->all->weights.stride_shift(0);
 
-  learner<svm_params>& l = init_learner(&params, learn, 1);
-  l.set_predict(predict);
+  learner<svm_params,example>& l = init_learner(params, learn, predict, 1);
   l.set_save_load(save_load);
   l.set_finish(finish);
   return make_base(l);
