@@ -3,4 +3,4 @@ Copyright (c) by respective owners including Yahoo!, Microsoft, and
 individual contributors. All rights reserved.  Released under a BSD (revised)
 license as described in the file LICENSE.
  */
-LEARNER::base_learner* kernel_svm_setup(vw &all);
+LEARNER::base_learner* kernel_svm_setup(arguments& arg);
