@@ -9,9 +9,9 @@ license as described in the file LICENSE.
 
 void learn(char&, LEARNER::base_learner&, example&) {}
 
-LEARNER::base_learner* noop_setup(vw& all)
+LEARNER::base_learner* noop_setup(arguments& arg)
 {
-  if (missing_option(all, true, "noop", "do no learning")) return nullptr;
+  if (arg.new_options("Noop Learner").critical("noop", "do no learning").missing()) return nullptr;
 
-  return &LEARNER::init_learner<char>(nullptr, learn, 1);
+  return make_base(LEARNER::init_learner(learn, 1));
 }
