@@ -11,7 +11,7 @@ vw& parse_args(int argc, char *argv[], trace_message_t trace_listener = nullptr,
 void parse_modules(vw& all, io_buf& model);
 void parse_sources(vw& all, io_buf& model, bool skipModelLoad = false);
 
-LEARNER::base_learner* setup_base(vw& all);
+LEARNER::base_learner* setup_base(arguments&);
 
 std::string spoof_hex_encoded_namespaces(const std::string& arg);
 // char** get_argv_from_string(string s, int& argc);
