@@ -53,6 +53,7 @@ namespace po = boost::program_options;
 #include "interactions.h"
 #include "vw_exception.h"
 #include "parse_example_json.h"
+#include "parse_dispatch_loop.h"
 
 using namespace std;
 
@@ -181,7 +182,7 @@ uint32_t cache_numbits(io_buf* buf, int filepointer)
     if (v_length == 0)
       THROW("cache version too short, cache file is probably invalid");
 
-    t.erase();
+    t.clear();
     if (t.size() < v_length)
       t.resize(v_length);
 
@@ -189,7 +190,7 @@ uint32_t cache_numbits(io_buf* buf, int filepointer)
     version_struct v_tmp(t.begin());
     if ( v_tmp != version )
     {
-      cout << "cache has possibly incompatible version, rebuilding" << endl;
+      //      cout << "cache has possibly incompatible version, rebuilding" << endl;
       t.delete_v();
       return 0;
     }
@@ -263,8 +264,8 @@ void reset_source(vw& all, size_t numbits)
 
       // close socket, erase final prediction sink and socket
       io_buf::close_file_or_socket(all.p->input->files[0]);
-      all.final_prediction_sink.erase();
-      all.p->input->files.erase();
+      all.final_prediction_sink.clear();
+      all.p->input->files.clear();
 
       sockaddr_in client_address;
       socklen_t size = sizeof(client_address);
@@ -329,7 +330,7 @@ void make_write_cache(vw& all, string &newname, bool quiet)
   io_buf* output = all.p->output;
   if (output->files.size() != 0)
   {
-    all.trace_message << "Warning: you tried to make two write caches.  Only the first one will be made." << endl;
+    all.opts_n_args.trace_message << "Warning: you tried to make two write caches.  Only the first one will be made." << endl;
     return;
   }
 
@@ -339,7 +340,7 @@ void make_write_cache(vw& all, string &newname, bool quiet)
   int f = output->open_file(temp.c_str(), all.stdin_off, io_buf::WRITE);
   if (f == -1)
   {
-    all.trace_message << "can't create cache file !" << endl;
+    all.opts_n_args.trace_message << "can't create cache file !" << endl;
     return;
   }
 
@@ -353,7 +354,7 @@ void make_write_cache(vw& all, string &newname, bool quiet)
   push_many(output->finalname,newname.c_str(),newname.length()+1);
   all.p->write_cache = true;
   if (!quiet)
-    all.trace_message << "creating cache_file = " << newname << endl;
+    all.opts_n_args.trace_message << "creating cache_file = " << newname << endl;
 }
 
 void parse_cache(vw& all, po::variables_map &vm, string source,
@@ -375,7 +376,7 @@ void parse_cache(vw& all, po::variables_map &vm, string source,
       {
         f = all.p->input->open_file(caches[i].c_str(), all.stdin_off, io_buf::READ);
       }
-      catch (exception e) { f = -1; }
+      catch (const exception& ) { f = -1; }
     if (f == -1)
       make_write_cache(all, caches[i], quiet);
     else
@@ -384,14 +385,14 @@ void parse_cache(vw& all, po::variables_map &vm, string source,
       if (c < all.num_bits)
       {
         if (!quiet)
-          all.trace_message << "WARNING: cache file is ignored as it's made with less bit precision than required!" << endl;
+          all.opts_n_args.trace_message << "WARNING: cache file is ignored as it's made with less bit precision than required!" << endl;
         all.p->input->close_file();
         make_write_cache(all, caches[i], quiet);
       }
       else
       {
         if (!quiet)
-          all.trace_message << "using cache_file = " << caches[i].c_str() << endl;
+          all.opts_n_args.trace_message << "using cache_file = " << caches[i].c_str() << endl;
         all.p->reader = read_cached_features;
         if (c == all.num_bits)
           all.p->sorted_cache = true;
@@ -406,7 +407,7 @@ void parse_cache(vw& all, po::variables_map &vm, string source,
   if (caches.size() == 0)
   {
     if (!quiet)
-      all.trace_message << "using no cache" << endl;
+      all.opts_n_args.trace_message << "using no cache" << endl;
     all.p->output->space.delete_v();
   }
 }
@@ -419,7 +420,7 @@ void parse_cache(vw& all, po::variables_map &vm, string source,
 void enable_sources(vw& all, bool quiet, size_t passes)
 {
   all.p->input->current = 0;
-  parse_cache(all, all.vm, all.data_filename, quiet);
+  parse_cache(all, all.opts_n_args.vm, all.data_filename, quiet);
 
   if (all.daemon || all.active)
   {
@@ -434,25 +435,25 @@ void enable_sources(vw& all, bool quiet, size_t passes)
     {
       stringstream msg;
       msg << "socket: " << strerror(errno);
-      all.trace_message << msg.str() << endl;
+      all.opts_n_args.trace_message << msg.str() << endl;
       THROW(msg.str().c_str());
     }
 
     int on = 1;
     if (setsockopt(all.p->bound_sock, SOL_SOCKET, SO_REUSEADDR, (char*)&on, sizeof(on)) < 0)
-      all.trace_message << "setsockopt SO_REUSEADDR: " << strerror(errno) << endl;
+      all.opts_n_args.trace_message << "setsockopt SO_REUSEADDR: " << strerror(errno) << endl;
 
     // Enable TCP Keep Alive to prevent socket leaks
     int enableTKA = 1;
     if (setsockopt(all.p->bound_sock, SOL_SOCKET, SO_KEEPALIVE, (char*)&enableTKA, sizeof(enableTKA)) < 0)
-      all.trace_message << "setsockopt SO_KEEPALIVE: " << strerror(errno) << endl;
+      all.opts_n_args.trace_message << "setsockopt SO_KEEPALIVE: " << strerror(errno) << endl;
 
     sockaddr_in address;
     address.sin_family = AF_INET;
     address.sin_addr.s_addr = htonl(INADDR_ANY);
     short unsigned int port = 26542;
-    if (all.vm.count("port"))
-      port = (uint16_t)all.vm["port"].as<size_t>();
+    if (all.opts_n_args.vm.count("port"))
+      port = (uint16_t)all.opts_n_args.vm["port"].as<size_t>();
     address.sin_port = htons(port);
 
     // attempt to bind to socket
@@ -464,34 +465,34 @@ void enable_sources(vw& all, bool quiet, size_t passes)
       THROWERRNO("listen");
 
     // write port file
-    if (all.vm.count("port_file"))
+    if (all.opts_n_args.vm.count("port_file"))
     {
       socklen_t address_size = sizeof(address);
       if (getsockname(all.p->bound_sock, (sockaddr*)&address, &address_size) < 0)
       {
-        all.trace_message << "getsockname: " << strerror(errno) << endl;
+        all.opts_n_args.trace_message << "getsockname: " << strerror(errno) << endl;
       }
       ofstream port_file;
-      port_file.open(all.vm["port_file"].as<string>().c_str());
+      port_file.open(all.opts_n_args.vm["port_file"].as<string>().c_str());
       if (!port_file.is_open())
-        THROW("error writing port file: " << all.vm["port_file"].as<string>());
+        THROW("error writing port file: " << all.opts_n_args.vm["port_file"].as<string>());
 
       port_file << ntohs(address.sin_port) << endl;
       port_file.close();
     }
 
     // background process (if foreground is not set)
-    if (!all.vm.count("foreground"))
+    if (!all.opts_n_args.vm.count("foreground"))
     {
       if (!all.active && daemon(1,1))
         THROWERRNO("daemon");
     }
 
     // write pid file
-    if (all.vm.count("pid_file"))
+    if (all.opts_n_args.vm.count("pid_file"))
     {
       ofstream pid_file;
-      pid_file.open(all.vm["pid_file"].as<string>().c_str());
+      pid_file.open(all.opts_n_args.vm["pid_file"].as<string>().c_str());
       if (!pid_file.is_open())
         THROW("error writing pid file");
 
@@ -576,7 +577,7 @@ child:
     socklen_t size = sizeof(client_address);
     all.p->max_fd = 0;
     if (!all.quiet)
-      all.trace_message << "calling accept" << endl;
+      all.opts_n_args.trace_message << "calling accept" << endl;
     int f = (int)accept(all.p->bound_sock,(sockaddr*)&client_address,&size);
     if (f < 0)
       THROWERRNO("accept");
@@ -589,7 +590,7 @@ child:
     all.p->input->files.push_back(f);
     all.p->max_fd = max(f, all.p->max_fd);
     if (!all.quiet)
-      all.trace_message << "reading data from port " << port << endl;
+      all.opts_n_args.trace_message << "reading data from port " << port << endl;
 
     all.p->max_fd++;
     if(all.active)
@@ -614,13 +615,13 @@ child:
     if (all.p->input->files.size() > 0)
     {
       if (!quiet)
-        all.trace_message << "ignoring text input in favor of cache input" << endl;
+        all.opts_n_args.trace_message << "ignoring text input in favor of cache input" << endl;
     }
     else
     {
       string temp = all.data_filename;
       if (!quiet)
-        all.trace_message << "Reading datafile = " << temp << endl;
+        all.opts_n_args.trace_message << "Reading datafile = " << temp << endl;
       try
       {
         all.p->input->open_file(temp.c_str(), all.stdin_off, io_buf::READ);
@@ -630,7 +631,7 @@ child:
         // when trying to fix this exception, consider that an empty temp is valid if all.stdin_off is false
         if (temp.size() != 0)
         {
-          all.trace_message << "can't open '" << temp << "', sailing on!" << endl;
+          all.opts_n_args.trace_message << "can't open '" << temp << "', sailing on!" << endl;
         }
         else
         {
@@ -638,7 +639,7 @@ child:
         }
       }
 
-      if (all.vm.count("json") || all.vm.count("dsjson"))
+      if (all.opts_n_args.vm.count("json") || all.opts_n_args.vm.count("dsjson"))
       {
         // TODO: change to class with virtual method
         if (all.audit)
@@ -654,7 +655,7 @@ child:
           all.p->jsonp = new json_parser<false>;
         }
 
-        all.p->decision_service_json = all.vm.count("dsjson") > 0;
+        all.p->decision_service_json = all.opts_n_args.vm.count("dsjson") > 0;
       }
       else
         all.p->reader = read_features_string;
@@ -668,15 +669,22 @@ child:
 
   all.p->input->count = all.p->input->files.size();
   if (!quiet && !all.daemon)
-    all.trace_message << "num sources = " << all.p->input->files.size() << endl;
+    all.opts_n_args.trace_message << "num sources = " << all.p->input->files.size() << endl;
+}
+
+void lock_done(parser& p)
+{
+  mutex_lock(&p.examples_lock);
+  p.done = true;
+  //in case get_example() is waiting for a fresh example, wake so it can realize there are no more.
+  condition_variable_signal_all(&p.example_available);
+  mutex_unlock(&p.examples_lock);
 }
 
 void set_done(vw& all)
 {
   all.early_terminate = true;
-  mutex_lock(&all.p->examples_lock);
-  all.p->done = true;
-  mutex_unlock(&all.p->examples_lock);
+  lock_done(*all.p);
 }
 
 void addgrams(vw& all, size_t ngram, size_t skip_gram, features& fs,
@@ -732,7 +740,7 @@ void generateGrams(vw& all, example* &ex)
     size_t length = ex->feature_space[index].size();
     for (size_t n = 1; n < all.ngram[index]; n++)
     {
-      all.p->gram_mask.erase();
+      all.p->gram_mask.clear();
       all.p->gram_mask.push_back((size_t)0);
       addgrams(all, n, all.skips[index], ex->feature_space[index],
                length, all.p->gram_mask, 0);
@@ -806,6 +814,7 @@ void setup_example(vw& all, example* ae)
     all.p->in_pass_counter++;
 
   ae->test_only = is_test_only(all.p->in_pass_counter, all.holdout_period, all.holdout_after, all.holdout_set_off, all.p->emptylines_separate_examples ? (all.holdout_period-1) : 0);
+  ae->test_only |= all.p->lp.test_label(&ae->l);
 
   if (all.p->emptylines_separate_examples && example_is_newline(*ae))
     all.p->in_pass_counter++;
@@ -817,7 +826,7 @@ void setup_example(vw& all, example* ae)
       if (all.ignore[*i])
       {
         //delete namespace
-        ae->feature_space[*i].erase();
+        ae->feature_space[*i].clear();
         memmove(i, i + 1, (ae->indices.end() - (i + 1))*sizeof(*i));
         ae->indices.end()--;
         i--;
@@ -945,7 +954,7 @@ primitive_feature_space* export_example(vw& all, example* ec, size_t& len)
 void releaseFeatureSpace(primitive_feature_space* features, size_t len)
 {
   for (size_t i = 0; i < len; i++)
-    delete features[i].fs;
+    delete[] features[i].fs;
   delete (features);
 }
 
@@ -956,25 +965,32 @@ void parse_example_label(vw& all, example&ec, string label)
   substring str = { cstr, cstr+label.length() };
   tokenize(' ', str, words);
   all.p->lp.parse_label(all.p, all.sd, &ec.l, words);
-  words.erase();
+  words.clear();
   words.delete_v();
 }
 
 void empty_example(vw& all, example& ec)
 {
   for (features& fs : ec)
-    fs.erase();
+    fs.clear();
 
-  ec.indices.erase();
-  ec.tag.erase();
+  ec.indices.clear();
+  ec.tag.clear();
   ec.sorted = false;
   ec.end_pass = false;
 }
 
-void finish_example(vw& all, example* ec)
+void finish_example(vw& all, multi_ex& ec_seq)
+{
+  for(auto ec : ec_seq)
+    finish_example(all, *ec);
+  ec_seq.clear();
+}
+
+void finish_example(vw& all, example& ec)
 {
   // only return examples to the pool that are from the pool and not externally allocated
-  if (!is_ring_example(all, ec))
+  if (!is_ring_example(all, &ec))
     return;
 
   mutex_lock(&all.p->output_lock);
@@ -982,11 +998,11 @@ void finish_example(vw& all, example* ec)
   condition_variable_signal(&all.p->output_done);
   mutex_unlock(&all.p->output_lock);
 
-  empty_example(all, *ec);
+  empty_example(all, ec);
 
   mutex_lock(&all.p->examples_lock);
-  assert(ec->in_use);
-  ec->in_use = false;
+  assert(ec.in_use);
+  ec.in_use = false;
   condition_variable_signal(&all.p->example_unused);
   if (all.p->done)
     condition_variable_signal_all(&all.p->example_available);
@@ -994,74 +1010,22 @@ void finish_example(vw& all, example* ec)
 }
 }
 
+void thread_dispatch(vw& all, v_array<example*> examples)
+{
+  mutex_lock(&all.p->examples_lock);
+  all.p->end_parsed_examples+=examples.size();
+  condition_variable_signal_all(&all.p->example_available);
+  mutex_unlock(&all.p->examples_lock);
+}
+
 #ifdef _WIN32
 DWORD WINAPI main_parse_loop(LPVOID in)
 #else
 void *main_parse_loop(void *in)
 #endif
 {
-  v_array<example*> examples = v_init<example*>();
   vw* all = (vw*)in;
-  size_t example_number = 0;  // for variable-size batch learning algorithms
-
-  try
-  {
-    size_t examples_available;
-    while(!all->p->done)
-    {
-      examples.push_back(&VW::get_unused_example(all)); // need at least 1 example
-      if (!all->do_reset_source && example_number != all->pass_length && all->max_examples > example_number
-          && all->p->reader(all, examples) > 0)
-      {
-        VW::setup_examples(*all, examples);
-        example_number+=examples.size();
-        examples_available=examples.size();
-      }
-      else
-      {
-        reset_source(*all, all->num_bits);
-        all->do_reset_source = false;
-        all->passes_complete++;
-
-        end_pass_example(*all, examples[0]);
-        if (all->passes_complete == all->numpasses && example_number == all->pass_length)
-        {
-          all->passes_complete = 0;
-          all->pass_length = all->pass_length*2+1;
-        }
-        if (all->passes_complete >= all->numpasses && all->max_examples >= example_number)
-        {
-          mutex_lock(&all->p->examples_lock);
-          all->p->done = true;
-          mutex_unlock(&all->p->examples_lock);
-        }
-        example_number = 0;
-        examples_available=1;
-      }
-      mutex_lock(&all->p->examples_lock);
-      all->p->end_parsed_examples+=examples_available;
-      condition_variable_signal_all(&all->p->example_available);
-      mutex_unlock(&all->p->examples_lock);
-      examples.erase();
-    }
-  }
-  catch (VW::vw_exception& e)
-  {
-    cerr << "vw example #" << example_number << "(" << e.Filename() << ":" << e.LineNumber() << "): " << e.what() << endl;
-  }
-  catch (exception& e)
-  {
-    cerr << "vw: example #" << example_number << e.what() << endl;
-  }
-
-  if (!all->p->done)
-  {
-    mutex_lock(&all->p->examples_lock);
-    all->p->done = true;
-    mutex_unlock(&all->p->examples_lock);
-  }
-
-  examples.delete_v();
+  parse_dispatch(*all, thread_dispatch);
   return 0L;
 }
 
@@ -1077,7 +1041,6 @@ example* get_example(parser* p)
       cout << "error: example should be in_use " << p->used_index << " " << p->end_parsed_examples << " " << ring_index << endl;
     assert((p->examples+ring_index)->in_use);
     mutex_unlock(&p->examples_lock);
-
     return p->examples + ring_index;
   }
   else
@@ -1126,6 +1089,20 @@ uint32_t* get_multilabel_predictions(example* ec, size_t& len)
   return labels.label_v.begin();
 }
 
+float get_action_score(example* ec, size_t i)
+{
+  ACTION_SCORE::action_scores scores = ec->pred.a_s;
+
+  if(i < scores.size()) {
+    return scores[i].score;
+  } else {
+    return 0.0;
+  }
+}
+
+size_t get_action_score_length(example* ec)
+{ return ec->pred.a_s.size(); }
+
 size_t get_tag_length(example* ec)
 {
   return ec->tag.size();
