@@ -8,8 +8,8 @@ license as described in the file LICENSE.
 
 namespace EntityRelationTask
 {
-void initialize(Search::search&, size_t&, po::variables_map&);
+void initialize(Search::search&, size_t&, arguments&);
 void finish(Search::search&);
-void run(Search::search&, std::vector<example*>&);
+void run(Search::search&, multi_ex&);
 extern Search::search_task task;
 }
