@@ -8,10 +8,10 @@ license as described in the file LICENSE.
 
 namespace GraphTask
 {
-void initialize(Search::search&, size_t&, po::variables_map&);
+void initialize(Search::search&, size_t&, arguments&);
 void finish(Search::search&);
-void setup(Search::search&, std::vector<example*>&);
-void run(Search::search&, std::vector<example*>&);
-void takedown(Search::search&, std::vector<example*>&);
+void setup(Search::search&, multi_ex&);
+void run(Search::search&, multi_ex&);
+void takedown(Search::search&, multi_ex&);
 extern Search::search_task task;
 }
